@@ -664,7 +664,6 @@ hex_document_write(HexDocument *doc)
 		fclose(file);
 		if(ret) {
 			doc->changed = FALSE;
-			undo_stack_free(doc);
 		}
 	}
 
@@ -979,6 +978,13 @@ hex_document_undo(HexDocument *doc)
 	return TRUE;
 }
 
+gboolean
+hex_document_is_writable(HexDocument *doc)
+{
+	return (doc->file_name != NULL &&
+			access(doc->file_name, W_OK) == 0);
+}
+
 gboolean
 hex_document_redo(HexDocument *doc)
 {
